import java.util.ArrayList;
import java.util.Stack;
import java.util.TreeMap;
public class TestableObject extends TestableObjectBase
{
  @Override
  protected boolean triangleContainmentIsFaulty()
  {
    return false;
  }
  @Override
  protected boolean triangleContains(int x1, int y1, int x2, int y2, int x3, int y3, int xtest, int ytest)
  {
    // Just a correct implementation.
    return new Vector(xtest, ytest).pointInTriangle(new Vector(x1, y1), new Vector(x2, y2), new Vector(x3, y3));
  }
  private static class Vec3
  {
    public int x, y, z;
    public Vec3(int x, int y, int z)
    {
      this.x = x;
      this.y = y;
      this.z = z;
    }
    public Vec3 crossProduct(Vec3 other)
    {
      return new Vec3(
        y*other.z - z*other.y,
        z*other.x - x*other.z,
        x*other.y - y*other.x
      );
    }
    public int dotProduct(Vec3 other)
    {
      return x * other.x + y * other.y + z * other.z;
    }
  }
  private static class Vector
  {
    public int x, y;
    public Vector(int x, int y)
    {
      this.x = x;
      this.y = y;
    }
    private boolean sameSide(Vector p, Vector a, Vector b)
    {
      Vec3 cp1 = b.minus(a).to3d().crossProduct(this.minus(a).to3d());
      Vec3 cp2 = b.minus(a).to3d().crossProduct(p.minus(a).to3d());
      return cp1.dotProduct(cp2) >= 0;
    }
    private Vec3 to3d()
    {
      return new Vec3(x, y, 0);
    }
    private Vector minus(Vector other)
    {
      return new Vector(x - other.x, y - other.y);
    }
    public boolean pointInTriangle(Vector a, Vector b, Vector c)
    {
      return sameSide(a, b, c) && sameSide(b, a, c) && sameSide(c, a, b);
    }
  }
  
  @Override
  protected boolean editDistanceIsFaulty()
  {
    return false;
  }
  private static int min(int a,int b,int c){
    return(Math.min(Math.min(a,b),c));
  }
  @Override
  protected int editDistance(String s, String t)
  {
    // Just a correct implementation.
    int m=s.length();
    int n=t.length();
    int[][]d=new int[m+1][n+1];
    for(int i=0;i<=m;i++){
      d[i][0]=i;
    }
    for(int j=0;j<=n;j++){
      d[0][j]=j;
    }
    for(int j=1;j<=n;j++){
      for(int i=1;i<=m;i++){
        if(s.charAt(i-1)==t.charAt(j-1)){
          d[i][j]=d[i-1][j-1];
        }
        else{
          d[i][j]=min((d[i-1][j]+1),(d[i][j-1]+1),(d[i-1][j-1]+1));
        }
      }
    }
    return(d[m][n]);
  }
  
  @Override
  protected boolean kromFormulaIsFaulty()
  {
    return false;
  }
  @Override
  protected boolean kromFormula(int variables, KromClause[] formula)
  {
    // Just a correct implementation.
    isSatisfiable = true;
    ArrayList<Vertex> verticesList = new ArrayList<Vertex>();
    Vertex[] vertices = new Vertex[variables*2];
    for (KromClause clause : formula)
    {
      testExistence(variables, clause.getVar1(), verticesList, vertices);
      testExistence(variables, clause.getVar2(), verticesList, vertices);
      vertices[clause.getVar1() + (!clause.getIsPositive1() ? 0 : variables)].neighbors.add(vertices[clause.getVar2() + (clause.getIsPositive2() ? 0 : variables)]);
      vertices[clause.getVar2() + (!clause.getIsPositive2() ? 0 : variables)].neighbors.add(vertices[clause.getVar1() + (clause.getIsPositive1() ? 0 : variables)]);
    }
    stronglyConnectedComponents(verticesList);
    return isSatisfiable;
  }
  private void testExistence(int variables, int position, ArrayList<Vertex> verticesList, Vertex[] vertices)
  {
    if (vertices[position] == null)
    {
      vertices[position] = new Vertex(position, true);
      vertices[position+variables] = new Vertex(position, false);
      verticesList.add(vertices[position]);
      verticesList.add(vertices[position+variables]);
    }
  }
  private boolean isSatisfiable;
  private int index;
  private void stronglyConnectedComponents(ArrayList<Vertex> vertices)
  {
    index = 0;
    Stack<Vertex> s = new Stack<Vertex>();
    for (Vertex v : vertices)
    {
      if (v.index == -1)
      {
        strongConnect(v, s);
      }
    }
  }
  private void strongConnect(Vertex v, Stack<Vertex> s)
  {
    v.index = index;
    v.lowlink = index;
    index++;
    s.push(v);
    v.onStack = true;

    for (Vertex w : v.neighbors)
    {
      if (w.index == -1)
      {
        strongConnect(w, s);
        v.lowlink = Math.min(v.lowlink, w.lowlink);
      }
      else if (w.onStack)
      {
        v.lowlink = Math.min(v.lowlink, w.index);
      }
    }

    if (v.lowlink == v.index)
    {
      TreeMap<Integer, Boolean> scc = new TreeMap<Integer, Boolean>();
      Vertex w;
      do
      {
        w = s.pop();
        w.onStack = false;
        Boolean x = scc.get(w.var);
        if (x == null)
        {
          scc.put(w.var, w.positive);
        }
        else if (x != w.positive)
        {
          isSatisfiable = false;
        }
      }
      while (w != v);
    }
  }
  private static class Vertex
  {
    public int index = -1;
    public int lowlink = 1;
    public boolean onStack = false;
    public int var;
    public boolean positive;
    public Vertex(int var, boolean positive)
    {
      this.var = var;
      this.positive = positive;
    }
    public ArrayList<Vertex> neighbors = new ArrayList<Vertex>();
  }
}























































